#################################################################################
# Created       : Martin Foster
# Modified      : 05-May-2003
#################################################################################
#
# Spelling - Module part of Ethereal Realms, designed to verify text and return
#            a list of improperly spelled words.
# Copyright (C) 2000-2003  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 204 - 817, 5th Street NE
#                 Calgary, Alberta
#                 T2E 3W9
#
#################################################################################

package eRealms::Spel;						# Package name

use IO::Socket;							# Socket library


#################################################################################
# Sub-Routines
#################################################################################

#####################
# Constructor
#
# Allows to class to be exported and thus spawn objects as a result

sub new
{
	#####################
	# Data Members
	my $class    = shift;					# Self reference
	my $database = shift;					# Database handle

	my %system;						# System hash

	#####################
	# Program Area

	# Link with hash
	$database->GetHashSystem(\%system);

	# Generate self
	my $self     = {
		ADDR => $system{'SetSpelAddr'},
		PORT => $system{'SetSpelPort'},
		NIL  => $system{'SetSpelNil'}
	 };

	# Bless $self into designated class
	bless $self, $class;

	# Return self
	return $self;
}


####################
# Spelling Checker
#
# This is a fairly straight forward process.   In the sense that a user
# send over a string, and this process will strip all HTML tags that it
# can find and then write the improper words in the hash with
# corresponding answers.

sub Check
{
	####################
	# Data members
	my $self   = shift;					# Reference to self
	my $words  = shift;					# Hash reference
	my $str    = shift;					# Submitted string

	my $line;						# Capturered line
	my $socket;						# Socket handle
	my $res;						# Result retrieved
	my $word;						# Currently selected

	my %local;						# Local hash


	####################
	# Program area

	# Strip out HTML
	# Tags and special characters
	$str =~ s/<[^>]*>//gs;
	$str =~ s/\&.*?\;//gs;

	# Comments
	$str =~ s/<!-|-->//gs;
	

	# Segregate
	while ($str =~ /([A-Za-z']+)/gs)
	{
		# Assign word
		$word =  $1;
		$word =~ s/(^'|'$)//gs;

		# Keep track of wording
		$local{$word} = (defined($local{$word})) ? $local{$word} + 1: 1;
	}

	# Configure socket
	$socket = IO::Socket::INET->new(
		PeerAddr => $self->{'ADDR'},
		PeerPort => $self->{'PORT'},
		Proto    => 'tcp',
		Type     => SOCK_STREAM)
	  or die("Cannot connect to $self->{'ADDR'}:$self->{'PORT'}");


	# Loop and verify
	foreach $word (sort(keys(%local)))
	{
		# Safety
		next unless (defined($word));
		next unless ($word ne '');

		# Display word
		print $socket "$word\n";

		# Read pipe
		while ($line = <$socket>)
		{
			# Truncate
			chomp($line);

			# Determine use of line
			# Calculate and exit loop
			if ($line =~ /^([\#\&\*\+\-\?])/)
			{
				# Capture result
				$res = $1;

				# Word is mispelled
				if (($res eq '&')
				 or ($res eq '?'))
				{
					# Retreive comma delimited list
					($_, $words->{$word}) = split(/:\s/, $line);	
				}

				# Not found in dictionary
				elsif ($res eq '#')
				{
					# Indicate as not found
					$words->{$word} = $self->{'NIL'};
				}

				# Final execution
				last;
			}
		}	
	}

	# Send closing
	print $socket "0\n";

	# Cleanup
	close($socket);
}

1;								# Return true
